'''Lightweight tool that runs commands when you push to GitHub, Bitbucket, or Gitlab.

Just type the commands you'd run manually and let Sloth CI run them for you!
'''

__title__ = 'sloth-ci'
__description__ = 'Lightweight CI tool'
__version__ = '2.0.6'
__author__ = 'Konstantin Molchanov'
__author_email__ = 'moigagoo@live.com'
__license__ = 'MIT'
